<?php
/**
* 2007-2023 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2023 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/
if (!defined('_PS_VERSION_')) {
    exit;
}

class Afex_shipping extends CarrierModule
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'afex_shipping';
        $this->tab = 'shipping_logistics';
        $this->version = '2.0.0';
        $this->author = 'Achraf Ammar';
        $this->need_instance = 1;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Afex Shipping');
        $this->description = $this->l('Afex Shipping - Available only in Tunisia');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');

        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
    }

    public function install()
    {
        Configuration::deleteByName('AFEX_SHIPPING_LIVE_MODE');
        Configuration::deleteByName('AFEX_SHIPPING_CARRIER_ID');
        Configuration::deleteByName('AFEX_SHIPPING_ACCOUNT_API_KEY');
        Configuration::deleteByName('AFEX_SHIPPING_ACCOUNT_STATUS_TRIGGER');

        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $carrier = $this->addCarrier();
        $this->addZones($carrier);
        $this->addGroups($carrier);
        $this->addRanges($carrier);
        Configuration::updateValue('AFEX_SHIPPING_LIVE_MODE', false);
        include dirname(__FILE__) . '/sql/install.php';
        $states = $this->getTunisianStates();
        $countryId = 208;
        $zoneId = 9;

        foreach ($states as $state) {
            // Check if the state already exists for the country
            $stateExists = (bool) Db::getInstance()->getValue('SELECT COUNT(*) FROM `' . _DB_PREFIX_ . 'state`
            WHERE `name` = "' . pSQL($state['name']) . '" AND `id_country` = ' . (int) $countryId);

            // If the state does not exist, create it
            if (!$stateExists) {
                $new_state = new State();
                $new_state->name = $state['name'];
                $new_state->iso_code = $state['iso_code'];
                $new_state->id_country = $countryId;
                $new_state->id_zone = $zoneId;
                $new_state->active = true;
                $new_state->add();
            }
        }

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('displayBackOfficeHeader') &&
            $this->registerHook('actionOrderStatusPostUpdate') &&
            $this->registerHook('updateCarrier') &&
            $this->registerHook('actionGetAdminOrderButtons') &&
            $this->registerTab('AdminUpload', 'AFEX Import') &&
            $this->registerTab('AdminFileResults', 'AFEX Rapprochement');
    }

    private function registerTab($className, $tabName)
    {
        $tab = new Tab();
        $tab->class_name = $className;
        $tab->module = $this->name;
        $tab->id_parent = (int) Tab::getIdFromClassName('IMPROVE');
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $tabName;
        }
        return $tab->add();
    }

    public function uninstall()
    {
        $carrier_id = (int) Configuration::get('AFEX_SHIPPING_CARRIER_ID');
        $carrier = new Carrier($carrier_id);

        // Delete all carrier zones
        $carrier_zones = $carrier->getZones($carrier_id);
        foreach ($carrier_zones as $zone) {
            $carrier->deleteZone($zone['id_zone']);
        }

        /** 
        // Delete all carrier ranges
        $carrier_ranges = $carrier->getRanges($carrier_id);
        foreach ($carrier_ranges as $range) {
            $carrier->deleteDeliveryPrice($range['id_range_weight'], $range['id_range_price']);
        }

        // Delete all carrier zone groups
        $carrier_zone_groups = $carrier->getCarrierZoneGroups($carrier_id);
        foreach ($carrier_zone_groups as $zone_group) {
            $carrier->deleteZoneGroup($zone_group['id_zone']);
        }
        */

        // Delete the carrier
        $carrier = new Carrier($carrier_id);
        $carrier->delete();

        Configuration::deleteByName('AFEX_SHIPPING_LIVE_MODE');
        Configuration::deleteByName('AFEX_SHIPPING_CARRIER_ID');
        Configuration::deleteByName('AFEX_SHIPPING_ACCOUNT_API_KEY');
        Configuration::deleteByName('AFEX_SHIPPING_ACCOUNT_STATUS_TRIGGER');

        include dirname(__FILE__) . '/sql/uninstall.php';

        return parent::uninstall() && $this->unregisterTab('AdminUpload') && $this->unregisterTab('AdminFileResults');
    }

    private function unregisterTab($className)
    {
        $idTab = (int) Tab::getIdFromClassName($className);
        if ($idTab) {
            $tab = new Tab($idTab);
            return $tab->delete();
        }
        return true;
    }

    public function getContent()
    {
        if (((bool) Tools::isSubmit('submitAfex_shipping_Module')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output . $this->renderForm();
    }

    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAfex_shipping_Module';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        return $helper->generateForm([$this->getConfigForm()]);
    }

    protected function getConfigForm()
    {
        // Get all the order statuses
        $orderStatuses = OrderState::getOrderStates($this->context->language->id);
        // Get the list of carriers
        $carriersDb = Db::getInstance()->executeS('SELECT * FROM '._DB_PREFIX_.'carrier WHERE deleted = 0 AND active = 1');
        $carriers = [];
        foreach($carriersDb as $carrier) {
            $carriers[] = [
                'id' => $carrier['id_carrier'],
                'name' => $carrier['name'],
            ];
        }
        // Prepare the order status options array
        $orderStatusOptions = [];
        foreach ($orderStatuses as $orderStatus) {
            $orderStatusOptions[] = [
                'id' => $orderStatus['id_order_state'],
                'name' => $orderStatus['name'],
            ];
        }

        return [
            'form' => [
                'legend' => [
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'AFEX_SHIPPING_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'text',
                        'desc' => $this->l('Enter a valid AFEX ID'),
                        'name' => 'AFEX_SHIPPING_API_URL',
                        'label' => $this->l('ID'),
                    ],
                    [
                        'type' => 'text',
                        'desc' => $this->l('Enter a valid API KEY'),
                        'name' => 'AFEX_SHIPPING_ACCOUNT_API_KEY',
                        'label' => $this->l('API KEY'),
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('AFEX status trigger'),
                        'desc' => $this->l('Choose the order status that will trigger the sending of the order to AFEX'),
                        'name' => 'AFEX_SHIPPING_ACCOUNT_STATUS_TRIGGER',
                        'options' => [
                            'query' => $orderStatusOptions,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('AFEX paiement status trigger'),
                        'desc' => $this->l('Choose the order paiement status that will trigger the sending of the order to AFEX'),
                        'name' => 'AFEX_SHIPPING_ACCOUNT_PAIEMENT_STATUS_TRIGGER',
                        'options' => [
                            'query' => $orderStatusOptions,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('AFEX associated carrier'),
                        'desc' => $this->l('Choose the associated carrier to AFEX'),
                        'name' => 'AFEX_SHIPPING_CARRIER_ID',
                        'options' => [
                            'query' => $carriers,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    protected function getConfigFormValues()
    {
        return [
            'AFEX_SHIPPING_LIVE_MODE' => Configuration::get('AFEX_SHIPPING_LIVE_MODE', true),
            'AFEX_SHIPPING_API_URL' => Configuration::get('AFEX_SHIPPING_API_URL', null),
            'AFEX_SHIPPING_ACCOUNT_API_KEY' => Configuration::get('AFEX_SHIPPING_ACCOUNT_API_KEY', null),
            'AFEX_SHIPPING_ACCOUNT_STATUS_TRIGGER' => Configuration::get('AFEX_SHIPPING_ACCOUNT_STATUS_TRIGGER', null),
            'AFEX_SHIPPING_ACCOUNT_PAIEMENT_STATUS_TRIGGER' => Configuration::get('AFEX_SHIPPING_ACCOUNT_PAIEMENT_STATUS_TRIGGER', null),
            'AFEX_SHIPPING_CARRIER_ID' => Configuration::get('AFEX_SHIPPING_CARRIER_ID', null),
        ];
    }

    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();
        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    public function getOrderShippingCost($params, $shipping_cost)
    {
        return $shipping_cost;
    }

    public function getOrderShippingCostExternal($params)
    {
        return true;
    }

    protected function addCarrier()
    {
        $existingCarrierId = (int) Configuration::get('AFEX_SHIPPING_CARRIER_ID');

        // If carrier already exists, return it
        if ($existingCarrierId) {
            $existingCarrier = new Carrier($existingCarrierId);
            if (Validate::isLoadedObject($existingCarrier)) {
                return $existingCarrier;
            }
        }

        $carrier = new Carrier();

        $carrier->name = $this->l('AFEX Tunisie');
        $carrier->is_module = true;
        $carrier->active = 1;
        $carrier->range_behavior = 1;
        $carrier->need_range = 1;
        $carrier->shipping_external = true;
        $carrier->range_behavior = 0;
        $carrier->external_module_name = $this->name;
        $carrier->shipping_method = 2;

        foreach (Language::getLanguages() as $lang) {
            $carrier->delay[$lang['id_lang']] = $this->l('Home Delivery within 1-3 days (Domestic)');
        }

        if ($carrier->add() == true) {
            @copy(dirname(__FILE__) . '/views/img/carrier_image.jpg', _PS_SHIP_IMG_DIR_ . '/' . (int) $carrier->id . '.jpg');
            Configuration::updateValue('AFEX_SHIPPING_CARRIER_ID', (int) $carrier->id);
            return $carrier;
        }

        return false;
    }

    protected function addGroups($carrier)
    {
        $groups_ids = [];
        $groups = Group::getGroups(Context::getContext()->language->id);
        foreach ($groups as $group) {
            $groups_ids[] = $group['id_group'];
        }

        $carrier->setGroups($groups_ids);
    }

    protected function addRanges($carrier)
    {
        $range_price = new RangePrice();
        $range_price->id_carrier = $carrier->id;
        $range_price->delimiter1 = '0';
        $range_price->delimiter2 = '10000';
        $range_price->add();

        $range_weight = new RangeWeight();
        $range_weight->id_carrier = $carrier->id;
        $range_weight->delimiter1 = '0';
        $range_weight->delimiter2 = '10000';
        $range_weight->add();
    }

    protected function addZones($carrier)
    {
        $zones = Zone::getZones();
        $affectedZones = $carrier->getZones();
        $zoneIds = array_column($affectedZones, 'id_zone');
        foreach ($zones as $zone) {
            if (!in_array($zone['id_zone'], $zoneIds)) {
                $carrier->addZone($zone['id_zone']);
            }
        }
    }

    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/city-select.js');
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        $newOrderStatus = $params['newOrderStatus'];
        // Get the order ID from the parameters
        $orderId = $params['id_order'];

        // Only send the order information to the shipping company's API if the new order status is "confirmed"
        if ($newOrderStatus->id == Configuration::get('AFEX_SHIPPING_ACCOUNT_STATUS_TRIGGER', null)) {
            // Send the order information to the shipping company's API
            $this->sendOrderToShippingCompany($orderId);
        } else if ($newOrderStatus->id == Configuration::get('AFEX_SHIPPING_ACCOUNT_PAIEMENT_STATUS_TRIGGER', null)){
           // Send the order information to the shipping company's API
           $this->sendOrderToShippingCompany($orderId, true); 
        }
    }

    public function sendOrderToShippingCompany($order_id, $isPaid=false)
    {
        // Get the order object
        $order = new Order($order_id);
        // Extract the necessary order information
        $customer = new Customer($order->id_customer);
        $address = new Address($order->id_address_delivery);
        $products = $order->getProducts();
        $state = new State($address->id_state);
        $state_name = $state->name;

        $products_description = '';
        foreach ($products as $product) {
            $products_description .= $product['product_name'] . ' x ' . $product['product_quantity'] . ', ';
        }
        // Prepare the data in the format required by the shipping company's API
        $data = [
            'nom' => $customer->firstname . ' ' . $customer->lastname,
            'telephone1' => $address->phone,
            'telephone2' => $address->phone_mobile,
            'gouvernorat' => $state_name,
            'delegation' => $address->city,
            'adresse' => $address->address1 . ' ' . $address->address2 . ' ' . $address->postcode,
            'code_postal' => $address->postcode,
            'marchandise' => $products_description,
            'reference' => $order->reference,
            'paquets' => '1',
            'observation' => '',
            'type_envoi' => 'Livraison à domicile',
            'cod' => $isPaid ? 0 : $order->total_paid,
            'mode_reglement' => 'Seulement en espèces',
            'avec_decharge' => '1'
        ];

        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => Configuration::get('AFEX_SHIPPING_API_URL', null),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => json_encode($data),
            CURLOPT_HTTPHEADER => array(
                'Content-Type: application/text',
                'X-API-Key: ' . Configuration::get('AFEX_SHIPPING_ACCOUNT_API_KEY', null)  
            ),
        ));

        $response = curl_exec($curl);
        $responseCode = curl_getinfo($curl, CURLINFO_RESPONSE_CODE);
        curl_close($curl);

        // Process the response to verify the order was successfully sent
        $json = json_decode(str_replace("'", '"', $response));
        if ($responseCode == 200 && isset($json->barcode)) {
            $barcode = $json->barcode;
            $sql = 'INSERT INTO ' . _DB_PREFIX_ . 'afex_shipping (id_order, carrier_awb, status, last_update) VALUES (' . (int) $order_id . ', "' . pSQL($barcode) . '", "pick-up", NOW())';
            Db::getInstance()->execute($sql);
            $order->shipping_number = $barcode;
            $order->update();
        } else {
            // Handle any errors or exceptions
            throw new Exception($response);
        }
    }

    public function getTunisianStates()
    {
        return [
            ['iso_code' => 'TN1', 'name' => 'Ariana'],
            ['iso_code' => 'TN2', 'name' => 'Beja'],
            ['iso_code' => 'TN3', 'name' => 'Ben Arous'],
            ['iso_code' => 'TN4', 'name' => 'Bizerte'],
            ['iso_code' => 'TN5', 'name' => 'Gabes'],
            ['iso_code' => 'TN6', 'name' => 'Gafsa'],
            ['iso_code' => 'TN7', 'name' => 'Jendouba'],
            ['iso_code' => 'TN8', 'name' => 'Kairouan'],
            ['iso_code' => 'TN9', 'name' => 'Kasserine'],
            ['iso_code' => 'TN10', 'name' => 'Kebili'],
            ['iso_code' => 'TN11', 'name' => 'Kef'],
            ['iso_code' => 'TN12', 'name' => 'Mahdia'],
            ['iso_code' => 'TN13', 'name' => 'Manouba'],
            ['iso_code' => 'TN14', 'name' => 'Medenine'],
            ['iso_code' => 'TN15', 'name' => 'Monastir'],
            ['iso_code' => 'TN16', 'name' => 'Nabeul'],
            ['iso_code' => 'TN17', 'name' => 'Sfax'],
            ['iso_code' => 'TN18', 'name' => 'Sidi Bouzid'],
            ['iso_code' => 'TN19', 'name' => 'Siliana'],
            ['iso_code' => 'TN20', 'name' => 'Sousse'],
            ['iso_code' => 'TN21', 'name' => 'Tataouine'],
            ['iso_code' => 'TN22', 'name' => 'Tozeur'],
            ['iso_code' => 'TN23', 'name' => 'Tunis'],
            ['iso_code' => 'TN24', 'name' => 'Zaghouan'],
        ];
    }

    public function hookActionGetAdminOrderButtons(){
        return "<b>Hello</b>";
    }
}
